/*
    Bantumi
    Copyright 2005 - 2007 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "gui.h"
#include "tcpip.h"
#include <dns_sd.h>
#include "MultiplayerController.h"

#import <UIKit/UIKit.h>
#import <QuartzCore/QuartzCore.h>
#include <arpa/inet.h>

#define BLUETOOTH_SERVICE_CLASS 0x10273929

#define CONN_TYPE_KEY @"connectionType"
#define HOSTNAME_KEY @"hostname"
#define PORT_KEY @"port"

void logMsg(const char* str, ...);

void initGui() {
}

@interface ActionSheetController : NSObject <UIActionSheetDelegate> {
@public
	int button;
}
@end

@implementation ActionSheetController
- (id) init {
	if ((self = [super init])) {
		button = -1;
	}
	return self;
}
- (void) actionSheet: (UIActionSheet*) sheet clickedButtonAtIndex: (NSInteger) b {
	button = b;
}
- (void) alertView: (UIAlertView*) view clickedButtonAtIndex: (NSInteger) index {
	button = index;
}
@end

void showWarning(const char *str) {
	ActionSheetController* controller = [[ActionSheetController alloc] init];
	UIAlertView* view = [[UIAlertView alloc] initWithTitle: @"Warning" message: [NSString stringWithCString: str encoding: NSISOLatin1StringEncoding] delegate: controller cancelButtonTitle: nil otherButtonTitles: @"OK", nil];
	[view show];
	while (controller->button < 0)
		[[NSRunLoop currentRunLoop] runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
	[view release];
	[controller release];
}

/*
Connection* showHostJoinDialog(BOOL bluetooth) {
	NSString* message = nil;
	if (bluetooth)
		message = @"Choose bluetooth connection type";
	else
		message = @"Choose TCP/IP connection type";
	
	ActionSheetController* controller = [[ActionSheetController alloc] init];
	UIAlertView* view = [[UIAlertView alloc] initWithTitle: @"Multiplayer" message: message delegate: controller cancelButtonTitle: @"Cancel" otherButtonTitles: @"Host", @"Join", nil];
//	UIActionSheet* view = [[UIActionSheet alloc] initWithTitle: @"Multiplayer" delegate: controller cancelButtonTitle: @"Cancel" destructiveButtonTitle: nil otherButtonTitles: @"Host", @"Join", nil];
	[view show];
	while (controller->button < 0)
		[[NSRunLoop currentRunLoop] runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
//	[view showInView: [[UIApplication sharedApplication] keyWindow]];
	[view release];
	int button = controller->button;
	[controller release];

	if (button == 1) {
		if (bluetooth)
			return showBluetoothDialog(NO);
		else
			return showTcpDialog(NO);
	} else if (button == 2) {
		if (bluetooth)
			return showBluetoothDialog(YES);
		else
			return showTcpDialog(YES);
	}

	return NULL;
}
*/

@implementation MultiplayerController
- (void) updateState {
	UIView* progress = hostProgress;
	if (isClient)
		progress = joinProgress;
	
	id controls[] = { typeButton, connectButton, hostPort, joinPort, joinHostname };
	int numControls = sizeof(controls)/sizeof(controls[0]);
	
	BOOL hidden = NO;
	BOOL enabled = NO;
	if (conn) {
		hidden = NO;
		enabled = NO;
	} else {
		hidden = YES;
		enabled = YES;
	}
	
	[progress setHidden: hidden];
	[tableView setHidden: !hidden];
	for (int i = 0; i < numControls; i++)
		[controls[i] setEnabled: enabled];
}

- (Connection*) run {
	serviceRef = 0;
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	[typeButton setSelectedSegmentIndex: 1];
	services = [[NSMutableArray alloc] init];
	
	NSString* lastHost = [defaults stringForKey: HOSTNAME_KEY];
	if (lastHost)
		[joinHostname setText: lastHost];
	NSString* lastPort = [defaults stringForKey: PORT_KEY];
	if (lastPort) {
		[joinPort setText: lastPort];
		[hostPort setText: lastPort];
	}
	
	if ([defaults objectForKey: CONN_TYPE_KEY])
		[typeButton setSelectedSegmentIndex: [defaults integerForKey: CONN_TYPE_KEY]];
	[self switchType];
	
	UIWindow* window = [[UIApplication sharedApplication] keyWindow];
	UIView* container = [[window subviews] lastObject];
	UIView* prev = [[container subviews] lastObject];
	
//	[UIView beginAnimations: nil context: nil];
//	[UIView setAnimationDuration: 1];
//	[UIView setAnimationTransition: UIViewAnimationTransitionCurlUp forView: window cache: YES];
	
	CATransition* animation = [CATransition animation];
	[animation setType: kCATransitionMoveIn];
	[animation setSubtype: kCATransitionFromTop];
	[animation setDuration: 0.3];
	[animation setTimingFunction: [CAMediaTimingFunction functionWithName: kCAMediaTimingFunctionEaseInEaseOut]];
	// why does the background view fade to white at the end of the transition?
	
	[container addSubview: dialogView];
	[prev removeFromSuperview];
	
	[[container layer] addAnimation: animation forKey: @"animationKey"];
	
//	[UIView commitAnimations];
	
	done = NO;
	while (!done) {
		[[NSRunLoop currentRunLoop] runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.05]];
		if (conn) {
			const char* errString;
			int ready = conn->ready(&errString);
			if (ready < 0) {
				delete conn;
				conn = NULL;

				if (serviceRef)
					DNSServiceRefDeallocate(serviceRef);
				serviceRef = 0;

				[self updateState];
				showWarning(errString);
			} else if (ready > 0) {
				break;
			}
		}
	}
	
	animation = [CATransition animation];
	[animation setType: kCATransitionReveal];
	[animation setSubtype: kCATransitionFromBottom];
	[animation setDuration: 0.3];
	[animation setTimingFunction: [CAMediaTimingFunction functionWithName: kCAMediaTimingFunctionEaseInEaseOut]];
	[animation setDelegate: self];
	
	[container addSubview: prev];
	[dialogView removeFromSuperview];
	
	[[container layer] addAnimation: animation forKey: @"animationKey"];
	
	finished = NO;
	while (!finished)
		[[NSRunLoop currentRunLoop] runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.05]];
	
	if (isClient)
		[defaults setObject: [joinPort text] forKey: PORT_KEY];
	else
		[defaults setObject: [hostPort text] forKey: PORT_KEY];
	[defaults setObject: [joinHostname text] forKey: HOSTNAME_KEY];
	[defaults setInteger: [typeButton selectedSegmentIndex] forKey: CONN_TYPE_KEY];

	if (serviceRef)
		DNSServiceRefDeallocate(serviceRef);
	serviceRef = 0;
	[browser release];
	browser = nil;
	[services release];
	services = nil;

	return conn;
}

- (IBAction) cancel {
	if (conn) {
		delete conn;
		conn = NULL;

		if (serviceRef)
			DNSServiceRefDeallocate(serviceRef);
		serviceRef = 0;

		[self updateState];
	} else {
		done = YES;
	}
}

- (IBAction) connect {
	
	const char* errString;
	if (isClient) {
		int portNumber = [[joinPort text] intValue];
		const char* hostName = [[joinHostname text] cStringUsingEncoding: NSISOLatin1StringEncoding];
		conn = TCPConnection::startConnect(hostName, portNumber, &errString);
	} else {
		int portNumber = [[hostPort text] intValue];

		DNSServiceRegister(&serviceRef, 0, 0, NULL, "_bantumi._tcp", NULL, NULL, htons(portNumber), 0, NULL, NULL, NULL);

		conn = TCPConnection::startAccept(portNumber, &errString);
	}
	[self updateState];
}

- (void) animationDidStop: (CAAnimation*) animation finished: (BOOL) flag {
	finished = YES;
}

- (IBAction) switchType {
	isClient = [typeButton selectedSegmentIndex] == 1;
	if (isClient) {
		[hostView removeFromSuperview];
		[contentArea addSubview: joinView];
		[joinProgress setHidden: YES];
		[joinPort setText: [hostPort text]];
		[services removeAllObjects];
		[tableView reloadData];
		[browser release];
		browser = nil;
		browser = [[NSNetServiceBrowser alloc] init];
		[browser setDelegate: self];
		[browser searchForServicesOfType: @"_bantumi._tcp" inDomain: @""];
	} else {
		[browser release];
		browser = nil;
		[joinView removeFromSuperview];
		[contentArea addSubview: hostView];
		[hostProgress setHidden: YES];
		[hostPort setText: [joinPort text]];
	}
}

- (void) netServiceBrowser: (NSNetServiceBrowser*) browser didFindService: (NSNetService*) service moreComing: (BOOL) moreServicesComing {
	if (!tableView)
		return;
	[service retain];
	[service setDelegate: self];
	[service resolveWithTimeout: 5];
//	[services addObject: service];
//	[table reloadData];
}

- (void) netServiceDidResolveAddress: (NSNetService*) service {
	if (!tableView)
		return;
	[services addObject: service];
	[service release];
	[tableView reloadData];
}

- (void) netService: (NSNetService*) service didNotResolve: (NSDictionary*) errorDict {
	return;
}

- (void) netServiceBrowser: (NSNetServiceBrowser*) browser didRemoveService: (NSNetService*) service moreComing: (BOOL) moreServicesComing {
	if (!tableView)
		return;
	[services removeObject: service];
	[tableView reloadData];
}

- (UITableViewCell*) tableView: (UITableView*) tableView cellForRowAtIndexPath: (NSIndexPath*) indexPath {
	int rowIndex = indexPath.row;
	NSNetService* service = [services objectAtIndex: rowIndex];
	NSString* text = [service name];
	UITableViewCell* cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:@"MyIdentifier"] autorelease];
	cell.text = text;
	return cell;
}

- (NSInteger) tableView: (UITableView*) tableView numberOfRowsInSection: (NSInteger) section {
	return [services count];
}

- (BOOL) tableView: (UITableView*) tableView shouldIndentWhileEditingRowAtIndexPath: (NSIndexPath*) indexPath {
	return NO;
}

- (NSIndexPath*) tableView: (UITableView*) tableView targetIndexPathForMoveFromRowAtIndexPath: (NSIndexPath*) sourceIndexPath toProposedIndexPath: (NSIndexPath*) proposedDestinationIndexPath {
	return proposedDestinationIndexPath;
}

- (void) tableView: (UITableView*) table didSelectRowAtIndexPath: (NSIndexPath*) newIndexPath {
	unsigned int row = newIndexPath.row;
	[tableView deselectRowAtIndexPath: newIndexPath animated: NO];
	if (row < 0 || row >= [services count])
		return;
	NSNetService* service = [services objectAtIndex: row];
//	NSArray* addresses = [service addresses];
	NSString* hostName = [service hostName]; // >= tiger
	if (hostName)
		[joinHostname setText: hostName];
	[joinPort setText: [NSString stringWithFormat: @"%d", [service port]]]; // >= leopard
/*
	for (unsigned int i = 0; i < [addresses count]; i++) {
		NSData* data = [addresses objectAtIndex: i];
		struct sockaddr* addr = (struct sockaddr*) [data bytes];
		char name[70] = "";
		if (addr->sa_family == AF_INET) {
			struct sockaddr_in* sa = (struct sockaddr_in*) addr;
			if (!hostName) {
				if (inet_ntop(addr->sa_family, &sa->sin_addr, name, sizeof(name)))
					[joinHostname setText: [NSString stringWithCString: name encoding: NSISOLatin1StringEncoding]];
			}
			[joinPort setText: [NSString stringWithFormat: @"%d", ntohs(sa->sin_port)]];
			break;
		} else if (addr->sa_family == AF_INET6) {
			struct sockaddr_in6* sa = (struct sockaddr_in6*) addr;
			if (!hostName) {
				if (inet_ntop(addr->sa_family, &sa->sin6_addr, name, sizeof(name)))
					
					[joinHostname setText: [NSString stringWithCString: name encoding: NSISOLatin1StringEncoding]];
			}
			[joinPort setText: [NSString stringWithFormat: @"%d", ntohs(sa->sin6_port)]];
			break;
		}
	}
*/
}

@end


Connection* showMultiplayerDialog() {
//	return showHostJoinDialog(NO);
	NSArray* array = [[NSBundle mainBundle] loadNibNamed: @"MultiplayerDialog" owner: nil  options: nil];
	MultiplayerController* controller = nil;
	for (id obj in array) {
		if ([obj isKindOfClass: [MultiplayerController class]]) {
			controller = (MultiplayerController*) obj;
		}
	}

	if (!controller)
		return NULL;
	[controller retain];
	Connection* conn = [controller run];
	[controller release];
	return conn;
}

